@@ -1,16 +1,20 @@
-{-# LANGUAGE CApiFFI               #-}
-{-# LANGUAGE CPP                   #-}
-{-# LANGUAGE DefaultSignatures     #-}
-{-# LANGUAGE FlexibleContexts      #-}
-{-# LANGUAGE MagicHash             #-}
-{-# LANGUAGE MultiParamTypeClasses #-}
-{-# LANGUAGE PackageImports        #-}
-{-# LANGUAGE PolyKinds             #-}
-{-# LANGUAGE ScopedTypeVariables   #-}
-{-# LANGUAGE Trustworthy           #-}
-{-# LANGUAGE TypeFamilies          #-}
-{-# LANGUAGE UnliftedFFITypes      #-}
-
+{-# LANGUAGE BangPatterns               #-}
+{-# LANGUAGE CApiFFI                    #-}
+{-# LANGUAGE CPP                        #-}
+{-# LANGUAGE DefaultSignatures          #-}
+{-# LANGUAGE DerivingStrategies         #-}
+{-# LANGUAGE FlexibleContexts           #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE MagicHash                  #-}
+{-# LANGUAGE MultiParamTypeClasses      #-}
+{-# LANGUAGE PackageImports             #-}
+{-# LANGUAGE PolyKinds                  #-}
+{-# LANGUAGE QuantifiedConstraints      #-}
+{-# LANGUAGE ScopedTypeVariables        #-}
+{-# LANGUAGE StandaloneDeriving         #-}
+{-# LANGUAGE Trustworthy                #-}
+{-# LANGUAGE TypeFamilies               #-}
+{-# LANGUAGE UnliftedFFITypes           #-}
 {-# OPTIONS_GHC -fno-warn-deprecations #-}
 
 ------------------------------------------------------------------------
@@ -68,6 +72,8 @@ module Data.Hashable.Class
 import Control.Applicative    (Const (..))
 import Control.DeepSeq        (NFData (rnf))
 import Control.Exception      (assert)
+import Control.Monad.ST       (runST)
+import Data.Array.Byte        (ByteArray (..))
 import Data.Complex           (Complex (..))
 import Data.Fixed             (Fixed (..))
 import Data.Functor.Classes   (Eq1 (..), Eq2 (..), Ord1 (..), Show1 (..))
@@ -79,6 +85,7 @@ import Data.List              (foldl')
 import Data.Proxy             (Proxy)
 import Data.Ratio             (Ratio, denominator, numerator)
 import Data.String            (IsString (..))
+import Data.Tuple             (Solo (..))
 import Data.Unique            (Unique, hashUnique)
 import Data.Version           (Version (..))
 import Data.Void              (Void, absurd)
@@ -89,7 +96,6 @@ import GHC.Base               (ByteArray#)
 import GHC.Conc               (ThreadId (..))
 import GHC.Fingerprint.Type   (Fingerprint (..))
 import GHC.Word               (Word (..))
-import System.IO.Unsafe       (unsafeDupablePerformIO)
 import System.Mem.StableName  (StableName, hashStableName)
 import Type.Reflection        (SomeTypeRep (..), TypeRep)
 import Type.Reflection.Unsafe (typeRepFingerprint)
@@ -98,7 +104,6 @@ import qualified Data.Array.Byte                as AB
 import qualified Data.ByteString                as B
 import qualified Data.ByteString.Lazy           as BL
 import qualified Data.ByteString.Short.Internal as BSI
-import qualified Data.ByteString.Unsafe         as B
 import qualified Data.Functor.Product           as FP
 import qualified Data.Functor.Sum               as FS
 import qualified Data.IntMap                    as IntMap
@@ -129,44 +134,13 @@ import Foreign.C.Types (CInt (..))
 #endif
 #endif
 
-#ifdef VERSION_ghc_bignum
 import GHC.Exts        (Int (..), sizeofByteArray#)
 import GHC.Num.BigNat  (BigNat (..))
 import GHC.Num.Integer (Integer (..))
 import GHC.Num.Natural (Natural (..))
-#endif
 
-#ifdef VERSION_integer_gmp
 
-# if MIN_VERSION_integer_gmp(1,0,0)
-#  define MIN_VERSION_integer_gmp_1_0_0
-# endif
-
-import GHC.Exts                  (Int (..))
-import GHC.Integer.GMP.Internals (Integer (..))
-# if defined(MIN_VERSION_integer_gmp_1_0_0)
-import GHC.Exts                  (sizeofByteArray#)
-import GHC.Integer.GMP.Internals (BigNat (BN#))
-# endif
-#endif
-
-#ifndef VERSION_ghc_bignum
-import GHC.Natural (Natural (..))
-#endif
-
-#if MIN_VERSION_base(4,11,0)
 import GHC.Float (castDoubleToWord64, castFloatToWord32)
-#else
-import Foreign.Marshal.Utils (with)
-import Foreign.Ptr           (castPtr)
-import Foreign.Storable      (peek)
-#endif
-
-#if MIN_VERSION_base(4,16,0)
-import Data.Tuple (Solo (..))
-#elif MIN_VERSION_base(4,15,0)
-import GHC.Tuple (Solo (..))
-#endif
 
 -- filepath >=1.4.100 && <1.5 has System.OsString.Internal.Types module
 #if MIN_VERSION_filepath(1,4,100) && !(MIN_VERSION_filepath(1,5,0))
@@ -190,16 +164,9 @@ import qualified "filepath" System.OsString.Internal.Types as FP (OsString (..),
 import System.OsString.Internal.Types (OsString (..), PosixString (..), WindowsString (..))
 #endif
 
-#ifdef VERSION_base_orphans
-import Data.Orphans ()
-#endif
-
-#ifdef VERSION_ghc_bignum_orphans
-import GHC.Num.Orphans ()
-#endif
-
 import Data.Hashable.Imports
 import Data.Hashable.LowLevel
+import Data.Hashable.XXH3
 
 #include "MachDeps.h"
 
@@ -212,9 +179,15 @@ infixl 0 `hashWithSalt`
 --
 -- Minimal implementation: 'hashWithSalt'.
 --
--- /Note:/ the hash is not guaranteed to be stable across
--- library versions, operating systems or architectures.
--- For stable hashing use named hashes: SHA256, CRC32 etc.
+-- 'Hashable' is intended exclusively for use in in-memory data structures.
+-- .
+-- 'Hashable' does /not/ have a fixed standard.
+-- This allows it to improve over time.
+-- .
+-- Because it does not have a fixed standard, different computers or computers on different versions of the code will observe different hash values.
+-- As such, 'Hashable' is not recommended for use other than in-memory datastructures.
+-- Specifically, 'Hashable' is not intended for network use or in applications which persist hashed values.
+-- For stable hashing use named hashes: sha256, crc32, xxhash etc.
 --
 -- If you are looking for 'Hashable' instance in @time@ package,
 -- check [time-compat](https://hackage.haskell.org/package/time-compat)
@@ -276,7 +249,7 @@ newtype instance HashArgs One  a = HashArgs1 (Int -> a -> Int)
 class GHashable arity f where
     ghashWithSalt :: HashArgs arity a -> Int -> f a -> Int
 
-class Eq1 t => Hashable1 t where
+class (Eq1 t, forall a. Hashable a => Hashable (t a)) => Hashable1 t where
     -- | Lift a hashing function through the type constructor.
     liftHashWithSalt :: (Int -> a -> Int) -> Int -> t a -> Int
 
@@ -291,7 +264,7 @@ genericLiftHashWithSalt :: (Generic1 t, GHashable One (Rep1 t)) => (Int -> a ->
 genericLiftHashWithSalt = \h salt -> ghashWithSalt (HashArgs1 h) salt . from1
 {-# INLINE genericLiftHashWithSalt #-}
 
-class Eq2 t => Hashable2 t where
+class (Eq2 t, forall a. Hashable a => Hashable1 (t a)) => Hashable2 t where
     -- | Lift a hashing function through the binary type constructor.
     liftHashWithSalt2 :: (Int -> a -> Int) -> (Int -> b -> Int) -> Int -> t a b -> Int
 
@@ -389,6 +362,7 @@ instance Hashable Word32 where
     hashWithSalt = defaultHashWithSalt
 
 instance Hashable Word64 where
+    hash = fromIntegral
     hashWithSalt = hashWord64
 
 instance Hashable () where
@@ -407,38 +381,17 @@ instance Hashable Char where
     hash = fromEnum
     hashWithSalt = defaultHashWithSalt
 
-#if defined(MIN_VERSION_integer_gmp_1_0_0) || defined(VERSION_ghc_bignum)
 instance Hashable BigNat where
-    hashWithSalt salt (BN# ba) = hashByteArrayWithSalt ba 0 numBytes salt
-                                 `hashWithSalt` size
-      where
-        size     = numBytes `quot` SIZEOF_HSWORD
-        numBytes = I# (sizeofByteArray# ba)
-#endif
+    hashWithSalt salt (BN# ba) = hashWithSalt salt (ByteArray ba)
 
 instance Hashable Natural where
-#if defined(VERSION_ghc_bignum)
     hash (NS n)   = hash (W# n)
     hash (NB bn)  = hash (BN# bn)
 
     hashWithSalt salt (NS n)  = hashWithSalt salt (W# n)
     hashWithSalt salt (NB bn) = hashWithSalt salt (BN# bn)
-#else
-#if defined(MIN_VERSION_integer_gmp_1_0_0)
-    hash (NatS# n)   = hash (W# n)
-    hash (NatJ# bn)  = hash bn
-
-    hashWithSalt salt (NatS# n)   = hashWithSalt salt (W# n)
-    hashWithSalt salt (NatJ# bn)  = hashWithSalt salt bn
-#else
-    hash (Natural n) = hash n
-
-    hashWithSalt salt (Natural n) = hashWithSalt salt n
-#endif
-#endif
 
 instance Hashable Integer where
-#if defined(VERSION_ghc_bignum)
     hash (IS n)  = I# n
     hash (IP bn) = hash (BN# bn)
     hash (IN bn) = negate (hash (BN# bn))
@@ -446,46 +399,6 @@ instance Hashable Integer where
     hashWithSalt salt (IS n)  = hashWithSalt salt (I# n)
     hashWithSalt salt (IP bn) = hashWithSalt salt (BN# bn)
     hashWithSalt salt (IN bn) = negate (hashWithSalt salt (BN# bn))
-#else
-#if defined(VERSION_integer_gmp)
-# if defined(MIN_VERSION_integer_gmp_1_0_0)
-    hash (S# n)   = (I# n)
-    hash (Jp# bn) = hash bn
-    hash (Jn# bn) = negate (hash bn)
-
-    hashWithSalt salt (S# n)   = hashWithSalt salt (I# n)
-    hashWithSalt salt (Jp# bn) = hashWithSalt salt bn
-    hashWithSalt salt (Jn# bn) = negate (hashWithSalt salt bn)
-# else
-    hash (S# int) = I# int
-    hash n@(J# size# byteArray)
-        | n >= minInt && n <= maxInt = fromInteger n :: Int
-        | otherwise = let size = I# size#
-                          numBytes = SIZEOF_HSWORD * abs size
-                      in hashByteArrayWithSalt byteArray 0 numBytes defaultSalt
-                         `hashWithSalt` size
-      where minInt = fromIntegral (minBound :: Int)
-            maxInt = fromIntegral (maxBound :: Int)
-
-    hashWithSalt salt (S# n) = hashWithSalt salt (I# n)
-    hashWithSalt salt n@(J# size# byteArray)
-        | n >= minInt && n <= maxInt = hashWithSalt salt (fromInteger n :: Int)
-        | otherwise = let size = I# size#
-                          numBytes = SIZEOF_HSWORD * abs size
-                      in hashByteArrayWithSalt byteArray 0 numBytes salt
-                         `hashWithSalt` size
-      where minInt = fromIntegral (minBound :: Int)
-            maxInt = fromIntegral (maxBound :: Int)
-# endif
-#else
-    hashWithSalt salt = foldl' hashWithSalt salt . go
-      where
-        go n | inBounds n = [fromIntegral n :: Int]
-             | otherwise   = fromIntegral n : go (n `shiftR` WORD_SIZE_IN_BITS)
-        maxInt = fromIntegral (maxBound :: Int)
-        inBounds x = x >= fromIntegral (minBound :: Int) && x <= maxInt
-#endif
-#endif
 
 instance Hashable a => Hashable (Complex a) where
     {-# SPECIALIZE instance Hashable (Complex Double) #-}
@@ -511,11 +424,7 @@ instance Hashable Float where
         | isIEEE x =
             assert (sizeOf x >= sizeOf (0::Word32) &&
                     alignment x >= alignment (0::Word32)) $
-#if MIN_VERSION_base(4,11,0)
             hash (castFloatToWord32 x)
-#else
-            hash ((unsafeDupablePerformIO $ with x $ peek . castPtr) :: Word32)
-#endif
         | otherwise = hash (show x)
     hashWithSalt = defaultHashWithSalt
 
@@ -530,11 +439,7 @@ instance Hashable Double where
         | isIEEE x =
             assert (sizeOf x >= sizeOf (0::Word64) &&
                     alignment x >= alignment (0::Word64)) $
-#if MIN_VERSION_base(4,11,0)
             hash (castDoubleToWord64 x)
-#else
-            hash ((unsafeDupablePerformIO $ with x $ peek . castPtr) :: Word64)
-#endif
         | otherwise = hash (show x)
     hashWithSalt = defaultHashWithSalt
 
@@ -668,76 +573,88 @@ instance Hashable1 [] where
         step (SP s l) x   = SP (h s x) (l + 1)
 
 instance Hashable B.ByteString where
-    hashWithSalt salt bs = unsafeDupablePerformIO $
-                           B.unsafeUseAsCStringLen bs $ \(p, len) ->
-                           hashPtrWithSalt p (fromIntegral len) (hashWithSalt salt len)
+    hash bs = fromIntegral (xxh3_64bit_withSeed_bs bs 0)
+
+    hashWithSalt salt bs =
+        fromIntegral (xxh3_64bit_withSeed_bs bs (fromIntegral (hashWithSalt salt len)))
+      where
+        len = B.length bs
 
 instance Hashable BL.ByteString where
-    hashWithSalt salt = finalise . BL.foldlChunks step (SP salt 0)
+    hashWithSalt salt lbs = runST $ do
+        s <- xxh3_64bit_createState
+        xxh3_64bit_reset_withSeed s (fromIntegral salt)
+        len <- BL.foldrChunks (step s) return lbs 0
+        xxh3_64bit_update_w64 s len
+        digest <- xxh3_64bit_digest s
+        return (fromIntegral digest)
       where
-        finalise (SP s l) = hashWithSalt s l
-        step (SP s l) bs  = unsafeDupablePerformIO $
-                            B.unsafeUseAsCStringLen bs $ \(p, len) -> do
-                                s' <- hashPtrWithSalt p (fromIntegral len) s
-                                return (SP s' (l + len))
+        step s bs next !acc = do
+            xxh3_64bit_update_bs s bs
+            next (acc + fromIntegral (B.length bs))
 
 instance Hashable BSI.ShortByteString where
-    hashWithSalt salt sbs@(BSI.SBS ba) =
-        hashByteArrayWithSalt ba 0 (BSI.length sbs) (hashWithSalt salt (BSI.length sbs))
+    hash (BSI.SBS ba) = hash (ByteArray ba)
+    hashWithSalt salt (BSI.SBS ba) = hashWithSalt salt (ByteArray ba)
 
 #if HAS_OS_STRING_filepath || HAS_OS_STRING_os_string
 -- | @since 1.4.2.0
-instance Hashable PosixString where
-    hashWithSalt salt (PosixString s) = hashWithSalt salt s
+deriving newtype instance Hashable PosixString
 
 -- | @since 1.4.2.0
-instance Hashable WindowsString where
-    hashWithSalt salt (WindowsString s) = hashWithSalt salt s
+deriving newtype instance Hashable WindowsString
 
 -- | @since 1.4.2.0
-instance Hashable OsString where
-    hashWithSalt salt (OsString s) = hashWithSalt salt s
+deriving newtype instance Hashable OsString
 #endif
 
 #if HAS_OS_STRING_filepath && HAS_OS_STRING_os_string
-instance Hashable FP.PosixString where
-    hashWithSalt salt (FP.PosixString s) = hashWithSalt salt s
-
-instance Hashable FP.WindowsString where
-    hashWithSalt salt (FP.WindowsString s) = hashWithSalt salt s
-
-instance Hashable FP.OsString where
-    hashWithSalt salt (FP.OsString s) = hashWithSalt salt s
+deriving newtype instance Hashable FP.PosixString
+deriving newtype instance Hashable FP.WindowsString
+deriving newtype instance Hashable FP.OsString
 #endif
 
 #if MIN_VERSION_text(2,0,0)
 
 instance Hashable T.Text where
+    hash (T.Text (TA.ByteArray arr) off len) =
+        fromIntegral (xxh3_64bit_withSeed_ba (ByteArray arr) off len 0)
     hashWithSalt salt (T.Text (TA.ByteArray arr) off len) =
-        hashByteArrayWithSalt arr off len (hashWithSalt salt len)
+        fromIntegral (xxh3_64bit_withSeed_ba (ByteArray arr) off len (fromIntegral (hashWithSalt salt len)))
 
 instance Hashable TL.Text where
-    hashWithSalt salt = finalise . TL.foldlChunks step (SP salt 0)
+    hashWithSalt salt lt = runST $ do
+        s <- xxh3_64bit_createState
+        xxh3_64bit_reset_withSeed s (fromIntegral salt)
+        len <- TL.foldrChunks (step s) return lt 0
+        xxh3_64bit_update_w64 s len
+        digest <- xxh3_64bit_digest s
+        return (fromIntegral digest)
       where
-        finalise (SP s l) = hashWithSalt s l
-        step (SP s l) (T.Text (TA.ByteArray arr) off len) = SP
-            (hashByteArrayWithSalt arr off len s)
-            (l + len)
+        step s (T.Text (TA.ByteArray arr) off len) next !acc = do
+            xxh3_64bit_update_ba s (ByteArray arr) off len
+            next (acc + fromIntegral len)
 
 #else
 
 instance Hashable T.Text where
+    hash (T.Text arr off len) =
+        fromIntegral (xxh3_64bit_withSeed_ba (ByteArray (TA.aBA arr)) (unsafeShiftL off 1) (unsafeShiftL len 1) 0)
     hashWithSalt salt (T.Text arr off len) =
-        hashByteArrayWithSalt (TA.aBA arr) (off `shiftL` 1) (len `shiftL` 1)
-        (hashWithSalt salt len)
+        fromIntegral (xxh3_64bit_withSeed_ba (ByteArray (TA.aBA arr)) (unsafeShiftL off 1) (unsafeShiftL len 1) (fromIntegral (hashWithSalt salt len)))
 
 instance Hashable TL.Text where
-    hashWithSalt salt = finalise . TL.foldlChunks step (SP salt 0)
+    hashWithSalt salt lt = runST $ do
+        s <- xxh3_64bit_createState
+        xxh3_64bit_reset_withSeed s (fromIntegral salt)
+        len <- TL.foldrChunks (step s) return lt 0
+        xxh3_64bit_update_w64 s len
+        digest <- xxh3_64bit_digest s
+        return (fromIntegral digest)
       where
-        finalise (SP s l) = hashWithSalt s l
-        step (SP s l) (T.Text arr off len) = SP
-            (hashByteArrayWithSalt (TA.aBA arr) (off `shiftL` 1) (len `shiftL` 1) s)
-            (l + len)
+        step s (T.Text arr off len) next !acc = do
+            xxh3_64bit_update_ba s (ByteArray (TA.aBA arr)) (unsafeShiftL off 1) (unsafeShiftL len 1)
+            next (acc + fromIntegral len)
 
 #endif
 
@@ -827,17 +744,14 @@ instance Hashable Version where
     hashWithSalt salt (Version branch tags) =
         salt `hashWithSalt` branch `hashWithSalt` tags
 
-instance Hashable (Fixed a) where
-    hashWithSalt salt (MkFixed i) = hashWithSalt salt i
+deriving newtype instance Hashable (Fixed a)
 
-instance Hashable a => Hashable (Identity a) where
-    hashWithSalt = hashWithSalt1
+deriving newtype instance Hashable a => Hashable (Identity a)
 instance Hashable1 Identity where
     liftHashWithSalt h salt (Identity x) = h salt x
 
 -- Using hashWithSalt1 would cause needless constraint
-instance Hashable a => Hashable (Const a b) where
-    hashWithSalt salt (Const x) = hashWithSalt salt x
+deriving newtype instance Hashable a => Hashable (Const a b)
 
 instance Hashable a => Hashable1 (Const a) where
     liftHashWithSalt = defaultLiftHashWithSalt
@@ -859,34 +773,24 @@ instance Hashable a => Hashable (NE.NonEmpty a) where
 instance Hashable1 NE.NonEmpty where
     liftHashWithSalt h salt (a NE.:| as) = liftHashWithSalt h (h salt a) as
 
-instance Hashable a => Hashable (Semi.Min a) where
-    hashWithSalt p (Semi.Min a) = hashWithSalt p a
-
-instance Hashable a => Hashable (Semi.Max a) where
-    hashWithSalt p (Semi.Max a) = hashWithSalt p a
+deriving newtype instance Hashable a => Hashable (Semi.Min a)
+deriving newtype instance Hashable a => Hashable (Semi.Max a)
 
 -- | __Note__: Prior to @hashable-1.3.0.0@ the hash computation included the second argument of 'Arg' which wasn't consistent with its 'Eq' instance.
 --
+-- Since @hashable-1.5.0.0@, @hash (Semi.arg a _) = hash a@
+--
 -- @since 1.3.0.0
 instance Hashable a => Hashable (Semi.Arg a b) where
+    hash (Semi.Arg a _) = hash a
     hashWithSalt p (Semi.Arg a _) = hashWithSalt p a
 
-instance Hashable a => Hashable (Semi.First a) where
-    hashWithSalt p (Semi.First a) = hashWithSalt p a
-
-
-instance Hashable a => Hashable (Semi.Last a) where
-    hashWithSalt p (Semi.Last a) = hashWithSalt p a
-
-
-instance Hashable a => Hashable (Semi.WrappedMonoid a) where
-    hashWithSalt p (Semi.WrapMonoid a) = hashWithSalt p a
-
+deriving newtype instance Hashable a => Hashable (Semi.First a)
+deriving newtype instance Hashable a => Hashable (Semi.Last a)
+deriving newtype instance Hashable a => Hashable (Semi.WrappedMonoid a)
 
 #if !MIN_VERSION_base(4,16,0)
-instance Hashable a => Hashable (Semi.Option a) where
-    hashWithSalt p (Semi.Option a) = hashWithSalt p a
-
+deriving newtype instance Hashable a => Hashable (Semi.Option a)
 #endif
 
 -- TODO: this instance is removed as there isn't Eq1 Min/Max, ...
@@ -912,10 +816,9 @@ instance Hashable a => Hashable (Semi.Option a) where
 -- instance Hashable1 Option where liftHashWithSalt h salt (Option a) = liftHashWithSalt h salt a
 #endif
 
--- | In general, @hash (Compose x) ≠ hash x@. However, @hashWithSalt@ satisfies
--- its variant of this equivalence.
-instance (Hashable1 f, Hashable1 g, Hashable a) => Hashable (Compose f g a) where
-    hashWithSalt = hashWithSalt1
+instance (Hashable (f (g a))) => Hashable (Compose f g a) where
+    hash (Compose x) = hash x
+    hashWithSalt p (Compose x) = hashWithSalt p x
 
 instance (Hashable1 f, Hashable1 g) => Hashable1 (Compose f g) where
     liftHashWithSalt h s = liftHashWithSalt (liftHashWithSalt h) s . getCompose
@@ -923,27 +826,31 @@ instance (Hashable1 f, Hashable1 g) => Hashable1 (Compose f g) where
 instance (Hashable1 f, Hashable1 g) => Hashable1 (FP.Product f g) where
     liftHashWithSalt h s (FP.Pair a b) = liftHashWithSalt h (liftHashWithSalt h s a) b
 
-instance (Hashable1 f, Hashable1 g, Hashable a) => Hashable (FP.Product f g a) where
-    hashWithSalt = hashWithSalt1
+instance (Hashable (f a), Hashable (g a)) => Hashable (FP.Product f g a) where
+    hashWithSalt s (FP.Pair a b) = s `hashWithSalt` a `hashWithSalt` b
 
 instance (Hashable1 f, Hashable1 g) => Hashable1 (FS.Sum f g) where
     liftHashWithSalt h s (FS.InL a) = liftHashWithSalt h (s `hashInt` 0) a
     liftHashWithSalt h s (FS.InR a) = liftHashWithSalt h (s `hashInt` distinguisher) a
 
-instance (Hashable1 f, Hashable1 g, Hashable a) => Hashable (FS.Sum f g a) where
-    hashWithSalt = hashWithSalt1
+instance (Hashable (f a), Hashable (g a)) => Hashable (FS.Sum f g a) where
+    hashWithSalt s (FS.InL a) = hashWithSalt (s `hashInt` 0) a
+    hashWithSalt s (FS.InR a) = hashWithSalt (s `hashInt` distinguisher) a
 
 -- | This instance was available since 1.4.1.0 only for GHC-9.4+
 --
 -- @since 1.4.2.0
 --
 instance Hashable AB.ByteArray where
-    hashWithSalt salt (AB.ByteArray ba) =
-        hashByteArrayWithSalt ba 0 numBytes salt
-        `hashWithSalt` size
+    hash ba@(AB.ByteArray ba') =
+        fromIntegral (xxh3_64bit_withSeed_ba ba 0 len 0)
       where
-        size     = numBytes `quot` SIZEOF_HSWORD
-        numBytes = I# (sizeofByteArray# ba)
+        !len = I# (sizeofByteArray# ba')
+
+    hashWithSalt salt ba@(AB.ByteArray ba') =
+        fromIntegral (xxh3_64bit_withSeed_ba ba 0 len (fromIntegral (hashWithSalt salt len)))
+      where
+        !len = I# (sizeofByteArray# ba')
 
 -------------------------------------------------------------------------------
 -- Hashed
@@ -1086,9 +993,7 @@ instance Hashable v => Hashable (Tree.Tree v) where
 -- Solo
 -------------------------------------------------------------------------------
 
-#if MIN_VERSION_base(4,15,0)
 instance Hashable a => Hashable (Solo a) where
     hashWithSalt = hashWithSalt1
 instance Hashable1 Solo where
     liftHashWithSalt h salt (Solo x) = h salt x
-#endif
