@@ -24,8 +24,8 @@ import GHC.Exts (unsafeCoerce#)
 import GHC.ST (ST(..), runST)
 import GHC.Word (Word8(..))
 import Test.QuickCheck hiding ((.&.))
-import Test.Framework (Test, testGroup)
-import Test.Framework.Providers.QuickCheck2 (testProperty)
+import Test.Tasty (TestTree, testGroup)
+import Test.Tasty.QuickCheck (testProperty)
 import GHC.Generics
 
 import qualified Data.ByteString.Short as BS
@@ -86,15 +86,16 @@ instance Arbitrary ChunkSize where
 
 -- | Ensure that the rechunk function causes a rechunked string to
 -- still match its original form.
-pTextRechunk :: T.Text -> NonEmptyList ChunkSize -> Bool
-pTextRechunk t cs = TL.fromStrict t == rechunkText t cs
+pTextRechunk :: T.Text -> NonEmptyList ChunkSize -> Property
+pTextRechunk t cs = TL.fromStrict t === rechunkText t cs
 
 -- | Lazy strings must hash to the same value no matter how they are
 -- chunked.
-pTextLazyRechunked :: T.Text
-                   -> NonEmptyList ChunkSize -> NonEmptyList ChunkSize -> Bool
-pTextLazyRechunked t cs0 cs1 =
-    hash (rechunkText t cs0) == hash (rechunkText t cs1)
+pTextLazyRechunked :: T.Text -> NonEmptyList ChunkSize -> NonEmptyList ChunkSize -> Property
+pTextLazyRechunked t cs0 cs1 = hash (rechunkText t cs0) === hash (rechunkText t cs1)
+
+pTextLazyRechunked' :: T.Text -> Int -> NonEmptyList ChunkSize -> NonEmptyList ChunkSize -> Property
+pTextLazyRechunked' t salt cs0 cs1 = hashWithSalt salt (rechunkText t cs0) === hashWithSalt salt (rechunkText t cs1)
 
 -- | Break up a string into chunks of different sizes.
 rechunkText :: T.Text -> NonEmptyList ChunkSize -> TL.Text
@@ -133,9 +134,11 @@ pBSRechunk t cs = fromStrict t == rechunkBS t cs
 
 -- | Lazy bytestrings must hash to the same value no matter how they
 -- are chunked.
-pBSLazyRechunked :: B.ByteString
-                 -> NonEmptyList ChunkSize -> NonEmptyList ChunkSize -> Bool
-pBSLazyRechunked t cs1 cs2 = hash (rechunkBS t cs1) == hash (rechunkBS t cs2)
+pBSLazyRechunked :: B.ByteString -> NonEmptyList ChunkSize -> NonEmptyList ChunkSize -> Property
+pBSLazyRechunked t cs1 cs2 = hash (rechunkBS t cs1) === hash (rechunkBS t cs2)
+
+pBSLazyRechunked' :: B.ByteString -> Int -> NonEmptyList ChunkSize -> NonEmptyList ChunkSize -> Property
+pBSLazyRechunked' t salt cs1 cs2 = hashWithSalt salt (rechunkBS t cs1) === hashWithSalt salt (rechunkBS t cs2)
 
 -- This wrapper is required by 'runST'.
 data ByteArray = BA { unBA :: ByteArray# }
@@ -222,7 +225,7 @@ instance (Arbitrary a, Hashable a) => Arbitrary (Hashed a) where
 pLiftedHashed :: Int -> Hashed (Either Int String) -> Bool
 pLiftedHashed s h = hashWithSalt s h == hashWithSalt1 s h
 
-properties :: [Test]
+properties :: [TestTree]
 properties =
     [ testProperty "bernstein" pHash
     , testGroup "text"
@@ -230,6 +233,7 @@ properties =
       , testProperty "text/lazy" pTextLazy
       , testProperty "text/rechunk" pTextRechunk
       , testProperty "text/rechunked" pTextLazyRechunked
+      , testProperty "text/rechunked-salt" pTextLazyRechunked'
       ]
     , testGroup "bytestring"
       [ testProperty "bytestring/strict" pBS
@@ -237,6 +241,7 @@ properties =
       , testProperty "bytestring/short" pBSShort
       , testProperty "bytestring/rechunk" pBSRechunk
       , testProperty "bytestring/rechunked" pBSLazyRechunked
+      , testProperty "bytestring/rechunked-salt" pBSLazyRechunked'
       ]
     , testGroup "generics"
       [
